import * as mc from '@minecraft/server';
import itemDataDrivenEvents from './_itemDataDrivenEvents_.js';

export default class itemsConfig {
    constructor(config) {
        this.is_using_item = "is_using_item";

        mc.world.afterEvents.itemStartUse.subscribe((eventData) => {
            let { itemStack, source } = eventData;
            let { id, itemUse } = config;
            if (itemStack?.typeId === id) {
                new itemDataDrivenEvents({ self: source }, itemUse);
            }
            source?.addTag(this.is_using_item);
            let tick = mc.system.runInterval(() => {
                if (!source?.hasTag(this.is_using_item)) {
                    mc.system.clearRun(tick);
                }
            }, 2);
        });

        mc.world.afterEvents.itemStopUse.subscribe((eventData) => {
            let { source } = eventData;
            mc.system.runTimeout(() => {
                source?.removeTag(this.is_using_item);
            }, 5);
        });

        mc.world.afterEvents.itemUseOn.subscribe((eventData) => {
            let { itemStack, source } = eventData;
            let { id, itemUseOn } = config;
            if (itemStack?.typeId === id) {
                new itemDataDrivenEvents({ self: source }, itemUseOn);
            }
        });

        mc.world.afterEvents.entityHitEntity.subscribe((eventData) => {
            let { damagingEntity, hitEntity } = eventData;
            let { id, itemHitEntity } = config;
            if (damagingEntity?.typeId === "minecraft:player") {
                let player = damagingEntity;
                let itemStack = player?.getComponent("minecraft:equippable")?.getEquipment("Mainhand");
                if (itemStack?.typeId === id) {
                    new itemDataDrivenEvents({ self: damagingEntity, other: hitEntity }, itemHitEntity);
                }
            }
        });

        mc.world.afterEvents.entityHitBlock.subscribe((eventData) => {
            let { damagingEntity, hitBlock } = eventData;
            let { id, itemHitBlock } = config;
            if (damagingEntity?.typeId === "minecraft:player") {
                let player = damagingEntity;
                let itemStack = player?.getComponent("minecraft:equippable")?.getEquipment("Mainhand");
                if (itemStack?.typeId === id) {
                    new itemDataDrivenEvents({ self: damagingEntity }, itemHitBlock);
                }
            }
        });
    }
}
