import { world, system } from "@minecraft/server";

export function createLine(dimension, from, to, particleId, amount) {
    for (let i = 0; i < amount; i++) {
        const divNum = i / amount;
        const particleLoc = {
            x: from.x - (from.x - to.x) * divNum,
            y: from.y - (from.y - to.y) * divNum,
            z: from.z - (from.z - to.z) * divNum
        };
        try {
            dimension.spawnParticle(particleId, particleLoc);
        } catch (e) {
            console.warn(`Couldn't spawn particle at ${particleLoc.x} ${particleLoc.y} ${particleLoc.z}`, e);
        }
    }
}

export function createLineThroughEntities(dimension) {
    const players = dimension.getPlayers().filter(player =>
        player.hasTag("alskill1")
    );

    if (players.length === 0) return;

    const entities = dimension.getEntities().filter(entity => 
        entity.typeId === "jjba:alastorskill1"
    );

    if (entities.length === 0) return;

    const playerLocation = { 
        ...players[0].location, 
        y: players[0].location.y + 0.9 
    };

    entities.forEach(entity => {
        const entityLocation = { 
            ...entity.location, 
            y: entity.location.y + 1.2 
        };
        createLine(dimension, playerLocation, entityLocation, "nika:alskill05", 20);
    });
}

const dimension = world.getDimension("overworld");
system.runInterval(() => {
    createLineThroughEntities(dimension);
}, 1);
