import { world, system } from "@minecraft/server";

export function createLine(dimension, from, to, particleId, amount) {
    for (let i = 0; i < amount; i++) {
        const divNum = i / amount;
        const particleLoc = {
            x: from.x - (from.x - to.x) * divNum,
            y: from.y - (from.y - to.y) * divNum,
            z: from.z - (from.z - to.z) * divNum
        };
        try {
            dimension.spawnParticle(particleId, particleLoc);
        } catch (e) {
            console.warn(`Couldn't spawn particle at ${particleLoc.x} ${particleLoc.y} ${particleLoc.z}`, e);
        }
    }
}

export function createLineThroughEntities(dimension) {
    const players = dimension.getPlayers().filter(player =>
        player.hasTag("shift") && player.hasTag("deal_active")
    );

    if (players.length === 0) return;

    // Filter entities to include only those with the "deal" tag and not the "deal_active" tag
    const entities = dimension.getEntities().filter(entity =>
        entity.hasTag("deal") && !entity.hasTag("deal_active")
    );

    if (entities.length > 0) {
        const playerLocation = { 
            ...players[0].location, 
            y: players[0].location.y + 0.9 
        };

        entities.forEach(entity => {
            const entityLocation = { 
                ...entity.location, 
                y: entity.location.y + 1.2 
            };
            createLine(dimension, playerLocation, entityLocation, "nika:alskill02", 20);
        });
    }
}

const dimension = world.getDimension("overworld");
system.runInterval(() => {
    createLineThroughEntities(dimension);
}, 1);
