import { world, system } from "@minecraft/server";

const tickRate = 1;
const maxEntityDistance = 50;

function modifyProjectileSpeed(projectile, speedMultiplier) {
    let velocity = projectile.getVelocity();
    if (!velocity || isNaN(velocity.x) || isNaN(velocity.y) || isNaN(velocity.z)) return;

    let originalVelocity = projectile.getDynamicProperty("originalVelocity");
    if (typeof originalVelocity !== "string") {
        projectile.setDynamicProperty("originalVelocity", JSON.stringify(velocity));
    }

    projectile.setDynamicProperty("modifiedVelocity", JSON.stringify({
        x: velocity.x * speedMultiplier,
        y: velocity.y,
        z: velocity.z * speedMultiplier
    }));
}

system.runInterval(() => {
    let activePlayers = world.getAllPlayers().filter(player => player.hasTag("mih_active"));
    if (activePlayers.length === 0) return;

    for (const player of activePlayers) {
        let speedMultiplier = 1;

        if (player.hasTag("mihsb")) speedMultiplier = 1.75;
        else if (player.hasTag("mih_taa")) speedMultiplier = 1.5;
        else if (player.hasTag("mih_ta")) speedMultiplier = 1.25;

        let dimension = player.dimension;
        let pos = player.location;

        for (let entity of dimension.getEntities()) {
            if (!entity.hasComponent("minecraft:projectile")) continue;

            const distance = Math.sqrt(
                Math.pow(entity.location.x - pos.x, 2) +
                Math.pow(entity.location.y - pos.y, 2) +
                Math.pow(entity.location.z - pos.z, 2)
            );

            if (distance > maxEntityDistance) continue;

            if (speedMultiplier > 1) {
                let storedVelocity = entity.getDynamicProperty("modifiedVelocity");

                if (!storedVelocity || typeof storedVelocity !== "string") {
                    modifyProjectileSpeed(entity, speedMultiplier);
                } else {
                    let parsedVelocity = JSON.parse(storedVelocity);
                    entity.applyImpulse({
                        x: parsedVelocity.x - entity.getVelocity().x,
                        y: 0,
                        z: parsedVelocity.z - entity.getVelocity().z
                    });
                }
            } else {
                let originalVelocity = entity.getDynamicProperty("originalVelocity");

                if (typeof originalVelocity === "string") {
                    let parsedOriginalVelocity = JSON.parse(originalVelocity);
                    entity.applyImpulse({
                        x: parsedOriginalVelocity.x - entity.getVelocity().x,
                        y: 0,
                        z: parsedOriginalVelocity.z - entity.getVelocity().z
                    });
                    entity.setDynamicProperty("originalVelocity", null);
                    entity.setDynamicProperty("modifiedVelocity", null);
                }
            }
        }
    }
}, tickRate);
