import { world, system } from "@minecraft/server";

const storedHealth = new Map();

function getEntityId(entity) {
    return entity.id || entity.nameTag || Math.random().toString(36).substring(2, 9);
}

system.runInterval(() => {
    const rt3Players = world.getPlayers().filter(player => player.hasTag("rt3"));
    if (rt3Players.length === 0) return;

    for (const player of rt3Players) {
        const dimensionEntities = player.dimension.getEntities();
        let rt22Exists = false;

        for (const entity of dimensionEntities) {
            if (entity.location && distance(player.location, entity.location) <= 100) {
                if (entity.hasTag("rt22")) {
                    rt22Exists = true;
                    storeEntityHealth(entity);
                } else {
                    restoreEntityHealth(entity);
                }
            }
        }

        if (!rt22Exists) {
            player.removeTag("rt3");
        }
    }
}, 20);

function storeEntityHealth(entity) {
    const entityId = getEntityId(entity);

    if (!storedHealth.has(entityId) && entity.hasComponent("minecraft:health")) {
        const healthComp = entity.getComponent("minecraft:health");
        storedHealth.set(entityId, healthComp.currentValue);
    }
}

function restoreEntityHealth(entity) {
    const entityId = getEntityId(entity);

    if (storedHealth.has(entityId) && entity.hasComponent("minecraft:health")) {
        const healthComp = entity.getComponent("minecraft:health");
        healthComp.setCurrentValue(storedHealth.get(entityId));
        storedHealth.delete(entityId);
    }
}

function distance(loc1, loc2) {
    return Math.sqrt(
        Math.pow(loc2.x - loc1.x, 2) +
        Math.pow(loc2.y - loc1.y, 2) +
        Math.pow(loc2.z - loc1.z, 2)
    );
}
