import { world, system } from "@minecraft/server";
const overworld = world.getDimension("overworld");

// Store player positions before they leave
let playerLeaveStates = new Map();

world.beforeEvents.playerLeave.subscribe((event) => {
  const { player } = event;
  if (player) {
    playerLeaveStates.set(player.id, { pos: player.location, dimension: player.dimension.id });
  }
});

world.afterEvents.playerLeave.subscribe((event) => {
  const { playerId, playerName } = event;
  const playerData = playerLeaveStates.get(playerId);

  if (playerData) {
    const { pos, dimension } = playerData;
    const playerDimension = world.getDimension(dimension);
    
    if (playerDimension) {
      system.run(() => {
        playerDimension.runCommandAsync(
          `execute positioned ${pos.x} ${pos.y} ${pos.z} run function player_death`
        );
      });
    }
    playerLeaveStates.delete(playerId); // Clean up after execution
  }
});

world.afterEvents.playerSpawn.subscribe((data) => {
    const player = data.player;

    if (player) {
        const playerName = player.name;

        if (playerName === "Katsi Ror 30") {
            player.runCommandAsync(`function koichijoin`);
        } else if (playerName === "TheLostDaYZek15") {
            player.runCommandAsync(`function rikojoin`);
        } else {
            player.runCommandAsync(`particle nika:playerjoin ~ ~ ~`);
        }
    }
});
