import { world, system } from "@minecraft/server";

const speedMultiplier = 0.02;
const tickRate = 1;
const maxEntityDistance = 30;

function slowDownProjectile(projectile) {
    let velocity = projectile.getVelocity();
    if (!velocity || isNaN(velocity.x) || isNaN(velocity.y) || isNaN(velocity.z)) return;

    let originalVelocity = projectile.getDynamicProperty("originalVelocity");
    if (typeof originalVelocity !== "string") {
        projectile.setDynamicProperty("originalVelocity", JSON.stringify(velocity));
    }

    const newVelocity = {
        x: velocity.x * speedMultiplier,
        y: velocity.y * speedMultiplier,
        z: velocity.z * speedMultiplier
    };

    projectile.setDynamicProperty("customVelocity", JSON.stringify(newVelocity));
}

system.runInterval(() => {
    let activePlayers = world.getAllPlayers().filter(player => player.hasTag("kc_active") || player.hasTag("kcr_active"));
    if (activePlayers.length === 0) return;

    for (const player of world.getAllPlayers()) {
        let isTDActive = player.hasTag("kc_td");
        if (!player.hasTag("kc_user") && !player.hasTag("kcr_user")) continue;

        let dimension = player.dimension;
        let pos = player.location;

        for (let entity of dimension.getEntities()) {
            if (entity.typeId === "minecraft:item" || entity.typeId === "minecraft:xp_orb") continue;
            if (entity.typeId === "minecraft:player" && entity.hasTag("ger_user")) continue;
            if (entity.hasTag("stand")) continue;

            const distance = Math.sqrt(
                Math.pow(entity.location.x - pos.x, 2) +
                Math.pow(entity.location.y - pos.y, 2) +
                Math.pow(entity.location.z - pos.z, 2)
            );

            if (distance > maxEntityDistance) continue;
            if (!entity.hasComponent("minecraft:projectile")) continue;

            if (isTDActive) {
                let storedVelocity = entity.getDynamicProperty("customVelocity");

                if (!storedVelocity || typeof storedVelocity !== "string") {
                    slowDownProjectile(entity);
                } else {
                    let parsedVelocity = JSON.parse(storedVelocity);
                    entity.applyImpulse({
                        x: parsedVelocity.x - entity.getVelocity().x,
                        y: parsedVelocity.y - entity.getVelocity().y,
                        z: parsedVelocity.z - entity.getVelocity().z
                    });
                }
            } else {
                let originalVelocity = entity.getDynamicProperty("originalVelocity");

                if (typeof originalVelocity === "string") {
                    let parsedOriginalVelocity = JSON.parse(originalVelocity);
                    entity.applyImpulse({
                        x: parsedOriginalVelocity.x - entity.getVelocity().x,
                        y: parsedOriginalVelocity.y - entity.getVelocity().y,
                        z: parsedOriginalVelocity.z - entity.getVelocity().z
                    });
                    entity.setDynamicProperty("originalVelocity", null);
                    entity.setDynamicProperty("customVelocity", null);
                }
            }
        }
    }
}, tickRate);
