import { world, system } from "@minecraft/server";

function spawnProjectile(player, projectileType, speed = 0.1) {
    const playerLocation = player.location;
    const playerDirection = player.getViewDirection();
    const dimension = player.dimension;

    const spawnLocation = {
        x: playerLocation.x + playerDirection.x,
        y: playerLocation.y + playerDirection.y + 1.5,
        z: playerLocation.z + playerDirection.z
    };

    const projectile = dimension.spawnEntity(projectileType, spawnLocation);

    if (projectile) {
        const impulse = {
            x: playerDirection.x * speed,
            y: playerDirection.y * speed,
            z: playerDirection.z * speed
        };

        projectile.applyImpulse(impulse);

        const intervalId = system.runInterval(() => {
            if (projectile.isValid()) {
                projectile.applyImpulse(impulse);
            } else {
                system.clearRun(intervalId);
            }
        }, 3);
    }
}

world.beforeEvents.worldInitialize.subscribe(initEvent => {
    initEvent.itemComponentRegistry.registerCustomComponent('jjba:use_skillspso', {
        onUse(e) {
            const player = e.source;
            const item = e.itemStack;

            item.getComponent('minecraft:cooldown').startCooldown(player);

            switch (item.typeId) {
                case 'jjba:spso_active_control':
                    player.runCommandAsync('execute as @s[tag=spso_user,tag=!tse] at @s run function spso_active');
                    break;
                case 'jjba:spso_disabled_control':
                    player.runCommandAsync('execute as @s[tag=spso_user,tag=!tse] run function spso_disabled');
                    break;
                case 'jjba:spso_stand_disc':
                    player.runCommandAsync('effect @s[tag=!stand_user] blindness 1 10');
                    player.runCommandAsync('effect @s[tag=!stand_user] night_vision 2 10 true');
                    player.runCommandAsync('execute as @s[tag=!stand_user] at @s run function spso_stand_up');
                    break;
                case 'jjba:spso_kh':
                    player.runCommandAsync('execute as @s[tag=!tse,tag=!unable_skills] at @s run function spso_kh');
                    break;
                case 'jjba:spso_fts':
                    player.runCommandAsync('execute as @s[tag=spso_user,tag=!unable_skills] at @s run function spso_fts');
                    spawnProjectile(player, 'jjba:gar', 1);
                    break;
                case 'jjba:spso_pb_control':
                    player.runCommandAsync('execute as @s[tag=!tse,tag=!unable_skills] run function spso_pb_active');
                    break;
                case 'jjba:spso_ts':
                    player.runCommandAsync('execute as @s[tag=!tse,tag=!unable_skills] at @s run function spso_ts_r');
                    
                    system.runTimeout(() => {
                    player.runCommandAsync('execute as @s[tag=spso_user] at @s run function spso_ts_notitle1');
                    }, 100); 
                    break;
                default:
                    return;
            }
        }
    });
});
