import { world, system } from "@minecraft/server";

function spawnProjectile(player, projectileType, speed = 1) {
    const playerLocation = player.location;
    const playerDirection = player.getViewDirection();
    const dimension = player.dimension;

    const spawnLocation = {
        x: playerLocation.x + playerDirection.x,
        y: playerLocation.y + playerDirection.y + 1.5,
        z: playerLocation.z + playerDirection.z
    };

    const projectile = dimension.spawnEntity(projectileType, spawnLocation);

    if (projectile) {
        const impulse = {
            x: playerDirection.x * speed,
            y: playerDirection.y * speed,
            z: playerDirection.z * speed
        };

        projectile.applyImpulse(impulse);

        const intervalId = system.runInterval(() => {
            if (projectile.isValid()) {
                projectile.applyImpulse(impulse);
            } else {
                system.clearRun(intervalId);
            }
        }, 1);
    }
}

world.beforeEvents.worldInitialize.subscribe(initEvent => {
    initEvent.itemComponentRegistry.registerCustomComponent('jjba:use_skillsw', {
        onUse(e) {
            const player = e.source;
            const item = e.itemStack;

            item.getComponent('minecraft:cooldown').startCooldown(player);

            switch (item.typeId) {
                case 'jjba:sw_active_control':
                    player.runCommandAsync('execute as @s[tag=sw_user,tag=!tse] at @s run function sw_active');
                    break;
                case 'jjba:sw_disabled_control':
                    player.runCommandAsync('execute as @s[tag=sw_user,tag=!tse] at @s run function sw_disabled');
                    break;
                case 'jjba:sw_b':
                    player.runCommandAsync('execute as @s[tag=sw_user,tag=!tse,tag=!shift,tag=!swgp1,tag=!unable_skills] at @s run function sw_b');
                    player.runCommandAsync('execute as @s[tag=sw_user,tag=!tse,tag=shift,tag=!swgp1,tag=!unable_skills] at @s run function sw_b1');
                    break;
                case 'jjba:sw_stand_disc':
                    player.runCommandAsync('effect @s[tag=!stand_user] blindness 1 10');
                    player.runCommandAsync('effect @s[tag=!stand_user] night_vision 2 10 true');
                    player.runCommandAsync('execute as @s[tag=!stand_user] at @s run function sw_stand_up');
                    break;
                case 'jjba:sw_h':
                    player.runCommandAsync('execute as @s[tag=sw_user,tag=!tse,tag=!shift,tag=!swgp1,tag=!unable_skills] at @s run function swh1');
                    player.runCommandAsync('execute as @s[tag=sw_user,tag=shift,tag=!tse,tag=!swgp1,tag=!unable_skills] at @s run function swh');
                    break;
                case 'jjba:sw_pb_control':
                    player.runCommandAsync('execute as @s[tag=sw_user,tag=!tse,tag=!swgp1,tag=!unable_skills] at @s run function sw_pb_active');
                    break;
                case 'jjba:swg':
                    player.runCommandAsync('execute as @s[tag=swg,tag=!tse,tag=!swgp1,tag=!unable_skills] at @s run function swg1');

                    system.runTimeout(() => {
                        spawnProjectile(player, 'jjba:gb', 1);
                    }, 60);

                    break;
                default:
                    return;
            }
        }
    });
});
