import { world, system } from "@minecraft/server";

const storedDamage = new Map();
const previousHealth = new Map();

function getEntityId(entity) {
    return entity.id || entity.nameTag || Math.random().toString(36).substring(2, 9);
}

system.runInterval(() => {
    const activePlayer = world.getPlayers().find(player => 
        player.hasTag("tw_active") || player.hasTag("sp_active") || player.hasTag("twoh_active") || player.hasTag("spso_active")
    );
    
    if (!activePlayer) return;

    const dimensionEntities = activePlayer.dimension.getEntities();

    for (const entity of dimensionEntities) {
        if (entity.hasComponent("minecraft:health")) {
            trackAndHandleDamage(entity);
        }
    }
}, 1);

function trackAndHandleDamage(entity) {
    const entityId = getEntityId(entity);
    const healthComp = entity.getComponent("minecraft:health");

    if (entity.hasTag("tse")) {
        if (!storedDamage.has(entityId)) {
            storedDamage.set(entityId, 0);
            previousHealth.set(entityId, healthComp.currentValue);
        }

        let lastHealth = previousHealth.get(entityId);
        let currentHealth = healthComp.currentValue;
        let damageTaken = lastHealth - currentHealth;

        if (damageTaken > 0) {
            storedDamage.set(entityId, storedDamage.get(entityId) + damageTaken);
            healthComp.setCurrentValue(lastHealth);
        }

        previousHealth.set(entityId, healthComp.currentValue);
    } else if (storedDamage.has(entityId)) {
        let totalDamage = storedDamage.get(entityId);
        let newHealth = Math.max(0, healthComp.currentValue - totalDamage);
        healthComp.setCurrentValue(newHealth);

        storedDamage.delete(entityId);
        previousHealth.delete(entityId);
    }
}
